Imports System.Data
Imports Microsoft.VisualBasic

Namespace AmazonServices

    Public Class AmazonJP

        Private Const SubscriptionId As String = "082Y49Q2WZZTJHHBN782" ' <-- v

        ''' <summary>
        ''' Searches book by isbn
        ''' </summary>
        Public Shared Function SearchBookByISBN(ByVal isbn As String) As DataView
            If (isbn Is Nothing) Then
                Return Nothing
            End If

            Dim service As New AWSECommerceService()
            Dim itemLookupRequest As New ItemLookupRequest
            With itemLookupRequest
                .ItemId = New String() {isbn}
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
            End With

            Dim itemLookup As New ItemLookup
            With itemLookup
                .SubscriptionId = SubscriptionId
                .Request = New ItemLookupRequest() {itemLookupRequest}
            End With

            Dim response As ItemLookupResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemLookup(itemLookup)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function


        ''' <summary>
        ''' Retrieves books in certain category from Web service
        ''' </summary>
        Public Shared Function SearchBooksByNode(ByVal CategoryId As String) As DataView
            Dim service As New AWSECommerceService()
            Dim searchRequest As New ItemSearchRequest
            With searchRequest
                .SearchIndex = "Books"
                .Sort = "salesrank"
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
                .BrowseNode = CategoryId
            End With

            Dim search As New ItemSearch
            With search
                .SubscriptionId = SubscriptionId
                .Request = New ItemSearchRequest() _
              {searchRequest}
            End With

            Dim response As ItemSearchResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemSearch(search)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function

        ''' <summary>
        ''' Searches for books by Author
        ''' </summary>
        Public Shared Function SearchBooksByTitle(ByVal Title As String) As DataView
            Dim Keywords As String = Nothing
            Dim Author As String = Nothing
            Dim PowerSearch As String = Nothing

            If (PowerSearch Is Nothing) AndAlso (Author Is Nothing) AndAlso _
              (Title Is Nothing) AndAlso (Keywords Is Nothing) Then
                Return Nothing
            End If

            Dim service As New AWSECommerceService()
            Dim searchRequest As New ItemSearchRequest
            With searchRequest
                .SearchIndex = "Books"
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
                If Not (PowerSearch Is Nothing) Then
                    .Power = PowerSearch
                Else
                    If Not (Author Is Nothing) Then
                        .Author = Author
                    End If
                    If Not (Title Is Nothing) Then
                        .Title = Title
                    End If
                    If Not (Keywords Is Nothing) Then
                        .Keywords = Keywords
                    End If
                End If
            End With

            Dim search As New ItemSearch
            With search
                .SubscriptionId = SubscriptionId
                .Request = New ItemSearchRequest() _
                  {searchRequest}
            End With

            Dim response As ItemSearchResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemSearch(search)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function

        ''' <summary>
        ''' Searches for books by Author
        ''' </summary>
        Public Shared Function SearchBooksByAuthor(ByVal Author As String) As DataView
            Dim Keywords As String = Nothing
            Dim Title As String = Nothing
            Dim PowerSearch As String = Nothing

            If (PowerSearch Is Nothing) AndAlso (Author Is Nothing) AndAlso _
              (Title Is Nothing) AndAlso (Keywords Is Nothing) Then
                Return Nothing
            End If

            Dim service As New AWSECommerceService()
            Dim searchRequest As New ItemSearchRequest
            With searchRequest
                .SearchIndex = "Books"
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
                If Not (PowerSearch Is Nothing) Then
                    .Power = PowerSearch
                Else
                    If Not (Author Is Nothing) Then
                        .Author = Author
                    End If
                    If Not (Title Is Nothing) Then
                        .Title = Title
                    End If
                    If Not (Keywords Is Nothing) Then
                        .Keywords = Keywords
                    End If
                End If
            End With

            Dim search As New ItemSearch
            With search
                .SubscriptionId = SubscriptionId
                .Request = New ItemSearchRequest() _
                  {searchRequest}
            End With

            Dim response As ItemSearchResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemSearch(search)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function

        ''' <summary>
        ''' Searches for books by keywords
        ''' </summary>
        Public Shared Function SearchBooksByKeyword(ByVal Keywords As String) As DataView
            Dim Author As String = Nothing
            Dim Title As String = Nothing
            Dim PowerSearch As String = Nothing

            If (PowerSearch Is Nothing) AndAlso (Author Is Nothing) AndAlso _
              (Title Is Nothing) AndAlso (Keywords Is Nothing) Then
                Return Nothing
            End If

            Dim service As New AWSECommerceService()
            Dim searchRequest As New ItemSearchRequest
            With searchRequest
                .SearchIndex = "Books"
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
                If Not (PowerSearch Is Nothing) Then
                    .Power = PowerSearch
                Else
                    If Not (Author Is Nothing) Then
                        .Author = Author
                    End If
                    If Not (Title Is Nothing) Then
                        .Title = Title
                    End If
                    If Not (Keywords Is Nothing) Then
                        .Keywords = Keywords
                    End If
                End If
            End With

            Dim search As New ItemSearch
            With search
                .SubscriptionId = SubscriptionId
                .Request = New ItemSearchRequest() _
                  {searchRequest}
            End With

            Dim response As ItemSearchResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemSearch(search)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function

        ''' <summary>
        ''' Searches for books by calling Amazon Web service
        ''' </summary>
        Public Shared Function SearchBooks(ByVal Author As String, _
          ByVal Title As String, ByVal Keywords As String, _
          ByVal PowerSearch As String) As DataView

            If (PowerSearch Is Nothing) AndAlso (Author Is Nothing) AndAlso _
              (Title Is Nothing) AndAlso (Keywords Is Nothing) Then
                Return Nothing
            End If

            Dim service As New AWSECommerceService()
            Dim searchRequest As New ItemSearchRequest
            With searchRequest
                .SearchIndex = "Books"
                .ResponseGroup = New String() {"Small", "Medium", "Large", "Images", "ItemAttributes", "OfferFull", "SalesRank", "Reviews"}
                If Not (PowerSearch Is Nothing) AndAlso PowerSearch.Length > 0 Then
                    .Power = PowerSearch
                Else
                    If Not (Author Is Nothing) AndAlso Author.Length > 0 Then
                        .Author = Author
                    End If
                    If Not (Title Is Nothing) AndAlso Title.Length > 0 Then
                        .Title = Title
                    End If
                    If Not (Keywords Is Nothing) AndAlso Keywords.Length > 0 Then
                        .Keywords = Keywords
                    End If
                End If
            End With

            Dim search As New ItemSearch
            With search
                .SubscriptionId = SubscriptionId
                .Request = New ItemSearchRequest() _
              {searchRequest}
            End With

            Dim response As ItemSearchResponse = Nothing
            Try
                service.Timeout = 5000
                response = service.ItemSearch(search)
            Catch
            End Try

            If (response Is Nothing) Then
                Return Nothing
            End If
            Return GetDataView(response.Items(0).Item)
        End Function

        Public Shared Function GetDataView(ByVal results() As Item) As DataView
            Dim dt As DataTable = CreateDataTable(1)
            If (Not (results) Is Nothing) Then
                Dim enumerator As System.Collections.IEnumerator = CType(results, System.Collections.IEnumerable).GetEnumerator
                Do While enumerator.MoveNext
                    Dim item As Item = CType(enumerator.Current, Item)
                    If (Not (item) Is Nothing) Then
                        Dim dataRow As DataRow = dt.NewRow
                        If (Not (item.ASIN) Is Nothing) Then
                            dataRow("ASIN") = item.ASIN
                        End If
                        If (Not (item.SalesRank) Is Nothing) Then
                            dataRow("SalesRank") = item.SalesRank
                        End If
                        If (Not (item.DetailPageURL) Is Nothing) Then
                            dataRow("AmazonURL") = item.DetailPageURL
                        End If
                        If Not (item.SmallImage Is Nothing) Then
                            dataRow("SmallImage") = item.SmallImage.URL
                        Else
                            dataRow("SmallImage") = "images/NoBookCover.jpg"
                        End If
                        If Not (item.MediumImage Is Nothing) Then
                            dataRow("MediumImage") = item.MediumImage.URL
                        Else
                            dataRow("SmallImage") = "images/NoBookCover.jpg"
                        End If
                        If Not (item.LargeImage Is Nothing) Then
                            dataRow("LargeImage") = item.LargeImage.URL
                        Else
                            dataRow("SmallImage") = "images/NoBookCover.jpg"
                        End If

                        If Not (item.ItemAttributes.Binding Is Nothing) Then
                            dataRow("Binding") = item.ItemAttributes.Binding
                        End If

                        If Not (item.ItemAttributes.PublicationDate Is Nothing) Then
                            dataRow("PublicationDate") = item.ItemAttributes.PublicationDate
                        Else
                            dataRow("PublicationDate") = "No PublicationDate"
                        End If

                        If Not (item.ItemAttributes.Publisher Is Nothing) Then
                            dataRow("Publisher") = item.ItemAttributes.Publisher
                        Else
                            dataRow("Publisher") = "No Publisher"
                        End If

                        If Not (item.ItemAttributes.ListPrice Is Nothing) Then
                            dataRow("ListPrice") = item.ItemAttributes.ListPrice.FormattedPrice
                        Else
                            dataRow("ListPrice") = "Not Listed"
                        End If

                        If Not (item.ItemAttributes.Title Is Nothing) Then
                            dataRow("Title") = item.ItemAttributes.Title
                        Else
                            dataRow("Title") = "No Title"
                        End If

                        If Not (item.ItemAttributes.Author Is Nothing) Then
                            dataRow("Author") = item.ItemAttributes.Author(0)
                        Else
                            dataRow("Author") = "No Author"
                        End If

                        If Not (item.Offers Is Nothing) Then
                            Dim offerArray As Offer() = item.Offers.Offer
                            If Not (offerArray Is Nothing) Then
                                If Not (offerArray(0).OfferListing Is Nothing) Then
                                    If Not (offerArray(0).OfferListing(0).Price Is Nothing) Then
                                        dataRow("AmazonPrice") = offerArray(0).OfferListing(0).Price.FormattedPrice
                                    Else
                                        dataRow("AmazonPrice") = "Not Listed"
                                    End If
                                End If
                            End If
                        End If

                        If Not (item.CustomerReviews Is Nothing) Then
                            dataRow("AverageRating") = item.CustomerReviews.AverageRating.ToString()
                            dataRow("TotalReviews") = item.CustomerReviews.TotalReviews.ToString()
                        End If
                        dt.Rows.Add(dataRow)
                    End If
                Loop
            End If
            Return New DataView(dt)
        End Function

        Public Shared Function CreateDataTable(ByVal seed As Integer) As DataTable
            Dim dt As New DataTable()
            Dim dc As DataColumn

            With dt.Columns
                dc = New DataColumn("AutoID", GetType(System.Int32))
                dc.AutoIncrement = True
                dc.AutoIncrementSeed = seed ' 1, 11, 21
                dc.AutoIncrementStep = 1
                .Add(dc)
                .Add(New DataColumn("ASIN", GetType(System.String)))
                .Add(New DataColumn("SmallImage", GetType(System.String)))
                .Add(New DataColumn("MediumImage", GetType(System.String)))
                .Add(New DataColumn("LargeImage", GetType(System.String)))

                .Add(New DataColumn("AmazonURL", GetType(System.String)))
                .Add(New DataColumn("Title", GetType(System.String)))
                .Add(New DataColumn("Author", GetType(System.String)))
                .Add(New DataColumn("ListPrice", GetType(System.String)))

                .Add(New DataColumn("SalesRank", GetType(System.String)))
                .Add(New DataColumn("PublicationDate", GetType(System.String)))
                .Add(New DataColumn("Publisher", GetType(System.String)))
                .Add(New DataColumn("Binding", GetType(System.String)))
                .Add(New DataColumn("AverageRating", GetType(System.String)))
                .Add(New DataColumn("TotalReviews", GetType(System.String)))

                .Add(New DataColumn("ProductGroup", GetType(System.String)))
                .Add(New DataColumn("AmazonPrice", GetType(System.String)))
            End With
            Return dt
        End Function

    End Class
End Namespace